use axum::extract::{FromRef, State};
use axum::{routing, Router};
use axum_template::{engine::Engine, Key, RenderHtml};
use minijinja::{path_loader, Environment};
use std::net::SocketAddr;

use std::sync::Arc;
use std::thread;
use std::time::Duration;

mod soundclips;
mod vbplay;

type AppEngine = Engine<Environment<'static>>;

#[derive(Clone)]
pub struct AppState {
    pub engine: AppEngine,
    pub clips: Vec<soundclips::SoundClip>,
}

impl FromRef<AppState> for AppEngine {
    fn from_ref(state: &AppState) -> Self {
        state.engine.clone()
    }
}

fn play_liv() {
    let file_name = "sounds/example/liv_aaaaaaaa.mp3";
    let tx = vbplay::audio_thread();

    tx.send(vbplay::Command::Play(file_name.into())).unwrap();

    thread::sleep(Duration::from_secs(5));

    tx.send(vbplay::Command::Stop).unwrap();
}

fn play_clip(file_name: &str) {
    let tx = vbplay::audio_thread();
    tx.send(vbplay::Command::Play(file_name.into())).unwrap();
}

#[tokio::main]
async fn main() {
    let mut jinja = Environment::new();
    jinja.set_loader(path_loader("templates"));
    let template_engine = Engine::from(jinja);

    let state = AppState {
        engine: template_engine,
        clips: soundclips::scan_directory_for_clips("E:/sounds/soundboard", &["mp3"])
            .expect("No Soundclips found."),
    };

    // Set the address to run our application on
    let addr = SocketAddr::from(([127, 0, 0, 1], 3000));

    // Build our application with a route
    let app = Router::new()
        .route("/", routing::get(home))
        .route("/play/:hash", routing::get(play_handler))
        .with_state(state);

    println!("listening on {}", addr);

    // Run it with hyper
    axum::Server::bind(&addr)
        .serve(app.into_make_service())
        .await
        .unwrap();
}

async fn home(engine: AppEngine, state: State<AppState>) -> impl axum::response::IntoResponse {
    RenderHtml(Key("index.html".to_owned()), engine, ())
}

async fn play_handler(
    axum::extract::Path(hash): axum::extract::Path<String>,
    state: State<AppState>,
) -> String {
    if let Some(clip) = state
        .clips
        .clone()
        .into_iter()
        .find(|s| hash == s.hash().to_string())
    {
        play_clip(clip.hash());
    }
    "".to_owned()
}
